/*!
    \file   df_access_control.h
    \author Bernd Ottmann
    \date   21.02.2003
    \brief  Definitionen der Funktionen zur Zutrittskontrolle Version 2.

    Für die Dokumentation der Listenfunktionen zur Zutrittskontrolle Version 2 \sa df_access_control_lists.h
*/
#ifndef DF_ACCESS_CONTROL_H
#define DF_ACCESS_CONTROL_H

#include "df_defines.h"

#if (DF_API_VERSION >= 0x04010400)

/*!
    \brief Führt Zutrittsbuchung via DLL aus.

    Die Funktion lässt sich aus Sicherheitsgründen nur auf Ausweise
    anwenden die im Feld "ActiveGeneral" der "Identification" Liste entweder
    den Wert "2" oder "9" eingetragen haben.

    \param connectionId Der zu verwendende Kanal.
    \param deviceAddress Die zu verwendende Busnummer.
    \param readerTM Der Wert der Zeichenfolge auf den dieser Zeiger verweist,
               muss dem Format des Feldes "TM" der "Reader" Liste folgen.
               Er muss demnach immer 3 Ziffern umfassen.
    \param identificationId Der Wert der Zeichenfolge auf den dieser Zeiger verweist,
               muss dem Format des Feldes "Id" der "Identification" Liste folgen.
			  Er darf maximal 20 Zeichen lang sein.
    \param identificationPin Der Wert der Zeichenfolge auf den dieser Zeiger verweist,
			   muss dem Format des Feldes "Pin" der "Identification" Liste folgen.
		       Er darf maximal 8 Ziffern beinhalten.
    \param statusText Zeiger auf Zeichenkette die nach erfolgreicher Abarbeitung der Funktion
				  den Status der Zutrittsprüfung erhält. Sie sollten mindestens 5 Zeichen vorsehen.
    \param statusTextLength Länge des reservierten Buffers/der übergebenen Zeichenkette.
    \param errorNumber Zeiger auf Variable die nach Fehler den Fehlercode erhaelt.

    \return Ergebnis der Ausfuehrung.
    \retval 0 Funktion konnte nicht ausgefuehrt werden.
    \retval 1 Funktion wurde erfolgreich ausgefuehrt.

    \version Ab DLL - Version 04.01.00 enthalten.
*/
DF_API_EXTERN int DF_API_CALL DFCEntrance2Identification( int connectionId, int deviceAddress, char *readerTM, char *identificationId, char *identificationPin, char *statusText, int *statusTextLength, int *errorNumber );

#endif

#if (DF_API_VERSION >= 0x04010500)
DF_API_EXTERN int DF_API_CALL DFCEntrance2OnlineAction( int connectionId, int deviceAddress, char *readerTM, int mask, int type, int duration, int *errorNumber);
#endif

#if (DF_API_VERSION >= 0x04030403)
DF_API_EXTERN int DF_API_CALL DFCAccessControlIdentification( int connectionId, int deviceAddress, int masterId, char *readerTM, char *identificationId, char *identificationPin, char *statusText, int *statusTextLength, int *errorNumber );
DF_API_EXTERN int DF_API_CALL DFCAccessControlOnlineAction( int connectionId, int deviceAddress, int masterId, char *readerTM, int mask, int type, int duration, int *errorNumber );
#endif
#if (DF_API_VERSION >= 0x04030500)
DF_API_EXTERN int DF_API_CALL DFCAccessControlKnobCommand( int connectionId, int deviceAddress, int knobCommand, int masterId, int knobNumber, char *params, char *buffer, int *bufferSize, int *errorNumber );
#endif

#if (DF_API_VERSION >= 0x04030600)
DF_API_EXTERN int DF_API_CALL DFCCustomCommand( int connectionId, int deviceAddress,int identification, int customCommand, char *buffer, int *bufferSize, int *errorNumber );
#endif

#endif
