/*!
    \file   df_defines.h
    \author Bernd Ottmann
    \date   21.02.2003
    \brief  Datei beinhaltet alle Defines die von der API benötigt werden.
    \note Um die API-Version einzustellen und damit die zur Verfügung stehenden Funktionen anzupassen /sa DF_API_VERSION
*/
#ifndef DF_DEFINES_H
#define DF_DEFINES_H

/*!
    \brief Version der verwendeten DLL.

    \todo Version der verwendeten DLL pruefen, um nur die enthaltenen Routinen zu 
    importieren oder auch zu exportieren.
*/
#define DF_API_VERSION          0x04030600

/*!
    \def DF_API_EXPORT
    \brief Gibt an ob die Funktionen exportiert oder importiert werden sollen.

    Wenn Sie die Header in Ihrem eigenen C/C++ Projekt verwenden, dann darf dieses Define nicht angegeben sein,
    dadurch wird ein import der Funktionen erzielt.
*/

// Eventuell fehlende typedefs ersetzen
#ifdef _MSC_VER
	#ifndef HANDLE
		//#define HANDLE void*
		typedef void* HANDLE;
	#endif
#else
	#ifndef HANDLE
                typedef int HANDLE;
	#endif
#endif

// ++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// +------------+ EXPORTIERTE FUNKTIONEN +----------------+
// ++++++++++++++++++++++++++++++++++++++++++++++++++++++++

/*!
 \def EXTERN_DECL Gibt den Praefix fuer Funktionen an. (Dekorierung und Export-, Importangabe)
*/

#ifdef DF_API_EXTERN
#undef DF_API_EXTERN
#endif

#ifdef _MSC_VER

#if defined( DF_API_EXPORT )
	// Einzubindende Bibliotheken: Version.lib, WSock32.lib
	#define DF_API_EXTERN         extern "C" __declspec(dllexport)
	// DllGetVersion
	#include "shlwapi.h"
#else
	#define DF_API_EXTERN         extern "C" __declspec(dllimport)
#endif

#define DF_API_CALL              __stdcall

#else

#define DF_API_EXTERN             extern "C"
#define DF_API_CALL

#endif

#endif
