/**
* @file  FileWlanConfig.h
* @brief Schnittstelle fuer die Klasse FileWlanConfig.
*/
//////////////////////////////////////////////////////////////////////
#ifndef FILEWLANCONFIG_H
#define FILEWLANCONFIG_H

#ifdef _MSC_VER
#pragma warning(disable : 4996)
#endif

// Verwaltungsklasse fuer Datei
#include "fileintelhex.h"
// 16bit/32bit CRC
#include "../Common/crc.h"
// Verschluesselung
#include "../Common/DF4Crypt.h"

// Strukturdefinitionen
#include "../Common/pzemaiv.h"

// Maximale Anzahl Datenbyte (2kByte)
#define MAX_MEM_WLANCONFIG   0x800

// Maximale Anzahl WlanConfigs
#define MAX_WLANCONFIG   8

#include <memory.h>

class FileWlanConfig : public FileIntelHex
{
public:
    FileWlanConfig();
    virtual ~FileWlanConfig();

public:
    unsigned long m_sizeData;

    // Verwaltungsstrukturen fuer WlanConfig
    WLANCONFIGNODE *m_pWlanConfigNode;
    WLANCONFIG     *m_pWlanConfig[MAX_WLANCONFIG];

    // Speicher fuer alle Daten
    unsigned char *m_memWlanConfig;

public:
    void reset();
    int Load(const char *fileName);
    int Save(const char *fileName, unsigned char append = 0);
    inline unsigned long GetSizeData() { return m_sizeData; };
    void Init(int iCount);
    static int DefaultWlanConfig(WLANCONFIG &wc);
    int SetWlanConfig(int idx, const WLANCONFIG &wc);
    WLANCONFIG GetWlanConfig(int idx);
    int GetWlanConfigCount()
    {
        return m_pWlanConfigNode->cnt;
    }
    void WlanConfigCRC(int idx);
    int WlanConfigCheckCRC(int idx);
    void WlanConfigCrypt(int idx);
    const char *fileName()
    {
        return _fileName;
    }
    void resetFileName() 
    {
        memset(_fileName, 0, sizeof(_fileName));
    }
    bool isModified() 
    {
        return (_crc != GetCRC32(m_memWlanConfig, MAX_MEM_WLANCONFIG));
    }

protected:
    int SaveWlanConfig(const char *fileName, unsigned char append = 0);

private:
    unsigned long _crc;
    char _fileName[260];
    CDF4Crypt _crypt;
};

#endif

